﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Client;
using TestPlanWordWriter.Common;
using Microsoft.TeamFoundation.TestManagement.Client;

namespace TestPlanWordWriter.UI
{
    public partial class CreateResultsControlForm : Form
    {
        public TfsTeamProjectCollection TeamProjectCollection { get; set; }
        public string TeamProjectName { get; set; }
        public int? TestPlan { get; set; }
        public string TestPlanName { get; set; }

        public TestResultsSettings ResultsSettings
        {
            get
            {
                TestResultsSettings settings = new TestResultsSettings();
                if (optMostRecent.Checked)
                {
                    settings.Options = TestResultsOptions.MostRecent;
                }
                else if (optSpecificBuild.Checked)
                {
                    settings.Options = TestResultsOptions.Build;
                    settings.SpecificBuildNumber = txtSpecificBuild.Text;
                }
                else if (optAllResults.Checked)
                {
                    settings.Options = TestResultsOptions.AllResults;
                }
                else
                {
                    throw new InvalidOperationException("Invalid state to get results settings");
                }
                return settings;
            }
        }

        public CreateResultsControlForm()
        {
            InitializeComponent();

            lstTestPlan.DisplayMember = "Name";
            lstTestPlan.ValueMember = "Id";
        }

        private void lstTestPlan_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstTestPlan.SelectedItem != null)
            {
                TestPlanListInfo currentItem = (TestPlanListInfo)lstTestPlan.SelectedItem;
                this.TestPlan = currentItem.Id;
                this.TestPlanName = currentItem.Name;
            }
            RefreshBtnOkEnabledState();
        }

        private void button1_Click(object sender, EventArgs e)
        {

        }

        private void CreateResultsControlForm_Load(object sender, EventArgs e)
        {
            ITestPlanCollection testPlans = TfsGateway.GetTestPlans(this.TeamProjectCollection, this.TeamProjectName);

            lstTestPlan.Items.Clear();

            foreach (ITestPlan testPlan in testPlans)
            {
                TestPlanListInfo testPlanItem = new TestPlanListInfo()
                {
                    Id = testPlan.Id,
                    Name = testPlan.Name
                };

                lstTestPlan.Items.Add(testPlanItem);
            }
            optMostRecent.Checked = true;
        }

        private void optMostRecent_CheckedChanged(object sender, EventArgs e)
        {
            RefreshBtnOkEnabledState();
        }

        private void optSpecificBuild_CheckedChanged(object sender, EventArgs e)
        {
            RefreshBtnOkEnabledState();
            RefreshTxtBuildSpecificState();
        }

        private void RefreshBtnOkEnabledState()
        {
            if (lstTestPlan.SelectedItem == null)
            {
                btnOK.Enabled = false;
            }
            else if (optSpecificBuild.Checked)
            {
                btnOK.Enabled = (!string.IsNullOrEmpty(txtSpecificBuild.Text));
            }
            else
            {
                btnOK.Enabled = true;
            }
        }

        private void RefreshTxtBuildSpecificState()
        {
            txtSpecificBuild.ReadOnly = (!optSpecificBuild.Checked);
        }

        private void optAllResults_CheckedChanged(object sender, EventArgs e)
        {
            RefreshBtnOkEnabledState();
            RefreshTxtBuildSpecificState();
        }

        private void txtSpecificBuild_TextChanged(object sender, EventArgs e)
        {
            RefreshBtnOkEnabledState();
            RefreshTxtBuildSpecificState();
        }
    }
}
